/*
    TCP Client
    - requests data from a TCP Server
    - deserializes received JSON Data

    by noiasca
    2024-03-07
*/

#include <WiFi.h>                              // comes with the ESP32 core
#include <ArduinoJson.h>                       // Install via library manager. Use a Version 6.x.x 

#define TXT_BOARDID "1"                        // an ID for the board
const uint16_t tcpInterval = 30;               // intervall to send data to a server in seconds. Set to 0 if you don't want to send data
const char* serverAddress = "172.18.67.218";   // the module will send information to that server/resource. Use an URI or a IP Adress
const uint16_t serverPort = 1040;              // 1040

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>                                   // if you have a config file with your credentials

#ifndef STASSID
#define STASSID "your-ssid"                                // set your SSID
#define STAPSK  "your-password"                            // set your wifi password
#endif

const char* ssid =     STASSID;
const char* password = STAPSK;

WiFiClient tcpClient;

//String input = "";

void tcpRequest() {
  Serial.print("TCP request");
  if (!tcpClient.connect(serverAddress, serverPort)) {
    Serial.println(" failed");
    return;
  }
  Serial.println(" connected");
  String request = "just give me your data please"; // the request can be a simple ? or meta information about the requested data or data from the client to be processed on the server
  tcpClient.print(request);
  String input = readResponse();

  // paste here the source code as provided from the ArduinoJson Assistant
  // https://arduinojson.org/v6/assistant/
  StaticJsonDocument<128> doc;

  DeserializationError error = deserializeJson(doc, input);

  if (error) {
    Serial.print("deserializeJson() failed: ");
    Serial.println(error.c_str());
    return;
  }

  uint16_t runtime = doc["runtime"];
  float temperature = doc["temperature"]; 
  float humidity = doc["humidity"]; 
  float pressure = doc["pressure"]; 

  // now you can process the received data:
  Serial.println("Server data");
  Serial.print("runtime:"); Serial.println(runtime);
  Serial.print("temperature:"); Serial.println(temperature);
  Serial.print("humidity:"); Serial.println(humidity);
  Serial.print("pressure:"); Serial.println(pressure);
}

void tcpTimer(uint32_t currentMillis = millis()) {
  static uint32_t previousMillis = 0;
  if (currentMillis - previousMillis > tcpInterval * 1000) {
    previousMillis = currentMillis;
    tcpRequest(); // trigger TCP request
  }
}

String readResponse() {
  uint32_t timeout = millis();
  while (tcpClient.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("E: Client Timeout !");
      tcpClient.stop();
      return "";
    }
  }

  String input;
  input.reserve(1000);
  // Read all the lines of the reply from server and print them to Serial
  while (tcpClient.available()) {
    String line = tcpClient.readStringUntil('\r');
    Serial.print(line);
    input += line;
  }

  Serial.printf("\nClosing connection\n\n");
  return input;
}

void handleSerial(){
  int c = Serial.read();
  if (c == 't') tcpRequest();
}

void setup() {
  Serial.begin(115200);
  while (!Serial) {
    delay(100);
  }

  Serial.println(F("\nTCP Client JSON "));
  Serial.print  (__DATE__);
  Serial.print  (' ');
  Serial.println(__TIME__);

  char myhostname[8] = {"esp"};
  strcat(myhostname, TXT_BOARDID);
  WiFi.hostname(myhostname);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

}

void loop() {
  tcpTimer();
  handleSerial();
  // do other things in the loop non blocking.
  // E.g. print out data to a display
}
//
